<?php

namespace App\Util;

use App\Exception;

use PHPMailer\PHPMailer\PHPMailer;
use PHPMailer\PHPMailer\Exception as MailerException;

class Util {
    private $connection;

    public function __construct (\Doctrine\DBAL\Connection $connection) {
        $this->connection = $connection;
    }
    
    public function newCode ($counterName, $size) {
        /* NOVOCODIGO */
        try {
            $nrseq = null;
            $connection = $this->connection;
            $storedProcedure = new StoredProcedure($connection, "NOVO_CODIGO_NT");
            $storedProcedure->addParam(new Param("P_CONTADOR", Param::PARAM_INPUT, $counterName, Param::PARAM_TYPE_STR, strlen($counterName)));
            $storedProcedure->addParam(new Param("P_SEQUENCIAL", Param::PARAM_OUTPUT, $nrseq, Param::PARAM_TYPE_INT, 20));
            $result = $storedProcedure->execute();
            return substr($result["P_SEQUENCIAL"], strlen($result["P_SEQUENCIAL"])-$size);
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        }
    }
    
    /**
     *
     *
     * @param StoredProcedure $procedure The procedure to be executed.
     * @param array $paramValues Param values to override values present in Param instance.
     *
     * @return array The output parameters indexed by name.
     */
    public function executeProcedure(StoredProcedure $procedure, array $paramValues = array()) {
        $queryStmt = $this->getQueryString($procedure);
        $stmt = $this->connection->prepare($queryStmt);
        $valuesOutput = array();
        foreach ($procedure->getParams() as $param) {
            if ($param->isOutput()) {
                $valuesOutput[$param->getName()] = $param->getValue();
                $stmt->bindParam($param->getParamAlias(), $valuesOutput[$param->getName()], $param->getType(), $param->getLength());
            } else {
                $stmt->bindValue($param->getParamAlias(), $param->getValue(), $param->getType());
            }
        }

        $stmt->execute();
        return $valuesOutput;
    }

    /**
     * @param StoredProcedure $procedure
     * @return string
     */
    protected function getQueryString(StoredProcedure $procedure) {
        $paramNames = array_map(function (Param $param) {
            return ":" . $param->getName();
        }, $procedure->getParams());
        $paramString = implode(", ", $paramNames);
        return "CALL " . $procedure->getName() . " ( " . $paramString . " )";
    }
    
    public function logError ($path, $content, $branch, $saleNumber) {
        $folder = $path  . "/" . date("Y") . "/" . date("m") . "/" . date("d") . "/";
        $fileName = $branch . "-" . $saleNumber.".txt";
        $currentFile = $folder . "/" . $fileName;
        
        if(!file_exists($folder)){
            mkdir($folder, 0775, true);
        }
        
        $line = date("d-m-Y H:i:s") . " - " . $content . PHP_EOL;
        file_put_contents($currentFile, $line, FILE_APPEND);
    }
    
    public function logSale ($path, $content, $branch, $saleseq) {
        $contentT = json_decode($content, true);
        
        $folder = $path  . "/" . date("Y") . "/" . date("m") . "/" . date("d") . "/";
        $fileName = $branch."-".$saleseq."-".round(microtime(true) * 1000).".json";
        $currentFile = $folder . "/" . $fileName;
        
        if(!file_exists($folder)){
            mkdir($folder, 0775, true);
        }
        
        file_put_contents($currentFile, $content);
    }
    
    public function logPDVClosing ($path, $content) {
        $contentT = json_decode($content, true);
        
        $folder = $path  . "/" . date("Y") . "/" . date("m") . "/" . date("d") . "/";
        $fileName = $contentT["numEmp"]."-".$contentT["numPDV"]."-".round(microtime(true) * 1000).".json";
        $currentFile = $folder . "/" . $fileName;
        
        if(!file_exists($folder)){
            mkdir($folder, 0775, true);
        }
        
        file_put_contents($currentFile, $content);
    }
    
    public function sendMail($recipient, $cc, $subject, $body, $params) {
        try {
            $configFile = file_get_contents(__DIR__ . "/../../configs/configs.json");
            $config     = json_decode($configFile, true);
        
            $mail = new PHPMailer(true);

            $mail->isSMTP();                            
            $mail->Host       = $config["host"];          
            $mail->SMTPAuth   = true;                   
            $mail->Username   = $config["username"];
            $mail->Password   = $this->decrypt($config["password"]); 
            $mail->SMTPSecure = $config["SMTPSecure"];    
            $mail->Port       = $config["port"];      
            $mail->CharSet    = "UTF-8";
            $mail->Encoding   = "base64";
            
            $mail->setFrom($config["username"], "Odhen Automação Comercial");

            if(empty($cc)) {
                //$cc = ["suporte.odhen@teknisa.com"];
                // $cc = ["daniel.fonseca@teknisa.com"];
            }

            foreach ($recipient as $r) { $mail->addAddress($r); }
            foreach ($cc as $c) { $mail->addCC($c); }
            
            $mail->isHTML(true);                                  
            $mail->Subject = $subject;
            $mail->Body    = str_replace(array_keys($params), array_values($params), $body);
            
            $mail->send();
        } catch (Exception $e) {
            var_dump("error: ".$e->getMessage());
        }
    }

    public function getPendingOrdersFile() {
        $configsFile  = file_get_contents(__DIR__ . "/../../configs/configs.json");
        $config       = json_decode($configsFile, true);

        $clientFolder = $this->connection->getParams();
        $clientFolder = (isset($clientFolder['dbname'])) ? $clientFolder['dbname'] : "undefined";
        
        $basePath     = $config['logPath'] . "/$clientFolder";

        $folder = $basePath."/dispatch";

        $fileName = "/pendingOrders.json";
        $currentFile = $folder . $fileName;
        
        if(!file_exists($folder)){
            mkdir($folder, 0775, true);
            file_put_contents($currentFile, json_encode([]));
        }

        return $currentFile;
    }
    
    public function getCurrentTimezone($state) {
        $timezones = [
            "AC" => "America/Rio_branco",   "AL" => "America/Maceio",
            "AP" => "America/Belem",        "AM" => "America/Manaus",
            "BA" => "America/Bahia",        "CE" => "America/Fortaleza",
            "DF" => "America/Fortaleza",    "ES" => "America/Fortaleza",
            "GO" => "America/Fortaleza",    "MA" => "America/Fortaleza",
            "MT" => "America/Cuiaba",       "MS" => "America/Campo_Grande",
            "MG" => "America/Fortaleza",    "PR" => "America/Fortaleza",
            "PB" => "America/Fortaleza",    "PA" => "America/Belem",
            "PE" => "America/Recife",       "PI" => "America/Fortaleza",
            "RJ" => "America/Fortaleza",    "RN" => "America/Fortaleza",
            "RS" => "America/Fortaleza",    "RO" => "America/Porto_Velho",
            "RR" => "America/Boa_Vista",    "SC" => "America/Fortaleza",
            "SE" => "America/Maceio",       "SP" => "America/Fortaleza",
            "TO" => "America/Araguaia",    
        ];
       
        return $timezones[$state];
    }
    
    private function decrypt($string) {
        $output = false;
        $encrypt_method = "AES-256-CBC";
    
        $secret_key = 'ZEEDHI';
        $secret_iv  = 'ZEEDHI IV';
    
        $key = hash('sha256', $secret_key);
        $iv  = substr(hash('sha256', $secret_iv), 0, 16);

        $output = openssl_decrypt(base64_decode($string), $encrypt_method, $key, 0, $iv);
        return $output;
    } 

    public function writeLog($type, $fileName, $content) {

        $configsFile  = file_get_contents(__DIR__ . "/../../configs/configs.json");
        $config       = json_decode($configsFile, true);

        $clientFolder = $this->connection->getParams();
        $clientFolder = (isset($clientFolder['dbname'])) ? $clientFolder['dbname'] : "undefined";

        $basePath     = $config['logPath'] . "/$clientFolder";

        if($type == 'order') {
            $folder      = $basePath . "/orders/" . date("Y") . "/" . date("m") . "/" . date("d") . "/";
            $currentFile = "$folder/$fileName";
            if(!file_exists($folder)) {
                mkdir($folder, 0775, true);
            }

            file_put_contents($currentFile, json_encode($content, JSON_PRETTY_PRINT | JSON_UNESCAPED_UNICODE));
        }
        
        if($type == 'order_update') {
            $folder      = $basePath . "/orders_update/" . date("Y") . "/" . date("m") . "/" . date("d") . "/";
            $currentFile = "$folder/$fileName";
            if(!file_exists($folder)) {
                mkdir($folder, 0775, true);
            }

            file_put_contents($currentFile, json_encode($content, JSON_PRETTY_PRINT | JSON_UNESCAPED_UNICODE));
        }
        
        if($type == 'sale') {
            $folder      = $basePath . "/sale/" . date("Y") . "/" . date("m") . "/" . date("d") . "/";
            $currentFile = "$folder/$fileName";
            if(!file_exists($folder)) {
                mkdir($folder, 0775, true);
            }

            file_put_contents($currentFile, json_encode($content, JSON_PRETTY_PRINT | JSON_UNESCAPED_UNICODE));
        }

        if($type == 'order_processed') {
            $folder      = $basePath . "/processed/" . date("Y") . "/" . date("m") . "/" . date("d") . "/";
            $currentFile = "$folder/$fileName";
            if(!file_exists($folder)) {
                mkdir($folder, 0775, true);
            }

            file_put_contents($currentFile, json_encode($content, JSON_PRETTY_PRINT | JSON_UNESCAPED_UNICODE));
        }

        if($type == 'error') {
            $folder      = $basePath . "/error/" . date("Y") . "/" . date("m") . "/" . date("d") . "/";
            $currentFile = "$folder/$fileName";
            if(!file_exists($folder)) {
                mkdir($folder, 0775, true);
            }

            file_put_contents($currentFile, json_encode($content, JSON_PRETTY_PRINT | JSON_UNESCAPED_UNICODE));
        }

        if($type == 'network') {
            $folder      = $basePath . "/networkError/" . date("Y") . "/" . date("m") . "/" . date("d") . "/";
            $currentFile = "$folder/$fileName";
            if(!file_exists($folder)) {
                mkdir($folder, 0775, true);
            }

            file_put_contents($currentFile, $content);
        }

        if($type == 'execution') {
            $folder      = $basePath . "/execution/" . date("Y") . "/" . date("m") . "/";
            $currentFile = "$folder" . date("d") . ".txt";
            if(!file_exists($folder)) {
                mkdir($folder, 0775, true);
            }

            file_put_contents($currentFile, $content, FILE_APPEND);
        }

        if($type == 'events') {
            $reference   = $content['correlationId'];
            $folder      = $basePath . "/events/" . date("Y") . "/" . date("m") . "/" . date("d") . "/$reference/";
            $currentFile = "$folder/$fileName";

            if(!file_exists($folder)) {
                mkdir($folder, 0775, true);
            }

	        file_put_contents($currentFile, json_encode($content, JSON_PRETTY_PRINT | JSON_UNESCAPED_UNICODE));
        }

        if($type == 'cancellation') {
            $folder      = $basePath . "/cancellation/" . date("Y") . "/" . date("m") . "/" . date("d") . "/";
            $currentFile = "$folder/$fileName";

            if(!file_exists($folder)) {
                mkdir($folder, 0775, true);
            }

	        file_put_contents($currentFile, json_encode($content, JSON_PRETTY_PRINT | JSON_UNESCAPED_UNICODE));
        }
    }
    
    public function formatNetworkError($networkData) {
        $eventTime = $networkData['eventTime'];
        $header    = implode($networkData['header'], "\r\n");
        $url       = $networkData['url'];
        $method    = $networkData['method'];
        $params    = is_array($networkData['params']) ? implode($networkData['params'], "\r\n") : $networkData['params'];
        $httpCode  = $networkData['httpCode'];
        $error     = $networkData['error'];
        $response  = $networkData['response'];

        $template = file_get_contents(__DIR__ . '/../../templates/networkError.txt');
        $templateParams = [
            ':EVENT'     => $eventTime,
            ':HEADER'    => $header,
            ':URL'       => $url,
            ':METHOD'    => $method,
            ':PARAMS'    => $params,
            ':HTTP_CODE' => $httpCode,
            ':ERROR'     => $error,
            ':RESPONSE'  => $response            
        ];

        $data = str_replace(array_keys($templateParams), array_values($templateParams), $template);
        return str_replace("\n", "\r\n", $data);
    }

    public function getMailList() {
        $list = [];

        $sql = "
            SELECT DSEMAILTEC 
              FROM LISTAEMAILTEC
             WHERE CDPROJETO = '013'
        ";
        
        $result = $this->connection->fetchAll($sql);
        
        if(empty($result)) {
            throw new Exception\GeneralErrorException("Cadastro de responsáveis técnicos inexistente.");
        }
        
        foreach($result as $row) {
            if(!empty($row['DSEMAILTEC']) && filter_var($row['DSEMAILTEC'], FILTER_VALIDATE_EMAIL)) {
                array_push($list, $row['DSEMAILTEC']);
            }
        }
        
        if(empty($list)) {
            throw new Exception\GeneralErrorException("Cadastro de responsáveis técnicos inexistente.");
        }
        
        return $list;
    }
}
